# Copyright (C) 2014 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero General Public License version 3.0 or later

from __future__ import print_function

import re
import sys


CHARIOT, \
HORSE, \
ELEPHANT, \
ADVISOR, \
KING, \
CANNON, \
PAWN, \
= range(7)

RED, \
BLACK, \
= range(2)

START_PARTY_RED = 'RED'
START_PARTY_BLACK = 'BLACK'


_SETUP_EXTRACTOR = re.compile('SETUP\\{([^}]+)}', re.MULTILINE)
_ITEM_ITERATOR = re.compile('(?P<put_piece>[RHEAKCPrheakcp][a-i][0-9])|(?P<move_offset>MOVE [1-9][0-9]*)|(?P<start_party>RED|BLACK)')
_FEN_ELEMENT_PATERN = '[RHEAKCPrheakcp1-9]+'
_FEN_EXTRACTOR = re.compile('^FEN[ \\t]+(?P<field_state>%s(?:/%s){9})( [rb])?' % (_FEN_ELEMENT_PATERN, _FEN_ELEMENT_PATERN), re.MULTILINE)


_PIECE_OF_UPPER_LETTER = {
    'R': CHARIOT,
    'H': HORSE,
    'E': ELEPHANT,
    'A': ADVISOR,
    'K': KING,
    'C': CANNON,
    'P': PAWN,
}


class PutPiece(object):
    def __init__(self, party, piece, x, y):
        self.party = party
        self.piece = piece
        self.x = x
        self.y = y

    def __str__(self):
        return 'PutPiece(party=%s, piece=%s, x=%d, y=%d)' \
                % (self.party, self.piece, self.x, self.y)


def _iterate_default_setup():
    for party, piece, x, y in (
            (RED, CHARIOT, 0, 0),
            (RED, HORSE, 1, 0),
            (RED, ELEPHANT, 2, 0),
            (RED, ADVISOR, 3, 0),
            (RED, KING, 4, 0),
            (RED, ADVISOR, 5, 0),
            (RED, ELEPHANT, 6, 0),
            (RED, HORSE, 7, 0),
            (RED, CHARIOT, 8, 0),

            (RED, CANNON, 1, 2),
            (RED, CANNON, 7, 2),

            (RED, PAWN, 0, 3),
            (RED, PAWN, 2, 3),
            (RED, PAWN, 4, 3),
            (RED, PAWN, 6, 3),
            (RED, PAWN, 8, 3),

            (BLACK, CHARIOT, 0, 9),
            (BLACK, HORSE, 1, 9),
            (BLACK, ELEPHANT, 2, 9),
            (BLACK, ADVISOR, 3, 9),
            (BLACK, KING, 4, 9),
            (BLACK, ADVISOR, 5, 9),
            (BLACK, ELEPHANT, 6, 9),
            (BLACK, HORSE, 7, 9),
            (BLACK, CHARIOT, 8, 9),

            (BLACK, CANNON, 1, 7),
            (BLACK, CANNON, 7, 7),

            (BLACK, PAWN, 0, 6),
            (BLACK, PAWN, 2, 6),
            (BLACK, PAWN, 4, 6),
            (BLACK, PAWN, 6, 6),
            (BLACK, PAWN, 8, 6),
            ):
        yield PutPiece(party, piece, x, y)


def iterate_wxf_tokens(filename):
    f = open(filename, 'r')
    content = f.read()
    f.close()

    fen_match = _FEN_EXTRACTOR.search(content)
    if fen_match:
        field_state_raw = fen_match.groupdict()['field_state']
        for i, line in enumerate(field_state_raw.split('/')):
            x = 0
            for char in line:
                try:
                    x += int(char)
                except ValueError:
                    party = BLACK if char.islower() else RED
                    piece = _PIECE_OF_UPPER_LETTER[char.upper()]
                    y = 9 - i
                    p = PutPiece(party, piece, x, y)
                    yield p
                    x += 1
        return

    setup_match = _SETUP_EXTRACTOR.search(content)
    if not setup_match:
        print('No custom setup found, assuming default setup.', file=sys.stderr)
        for put_piece in _iterate_default_setup():
            yield put_piece
        return

    setup_body = setup_match.group(1)
    for item_match in re.finditer(_ITEM_ITERATOR, setup_body):
        gd = item_match.groupdict()
        if gd['put_piece']:
            piece_letter, x_lower_alpha, y_0_9 = gd['put_piece']

            party = BLACK if piece_letter.islower() else RED
            piece = _PIECE_OF_UPPER_LETTER[piece_letter.upper()]
            x = ord(x_lower_alpha) - ord('a')
            y = int(y_0_9)

            yield PutPiece(party, piece, x, y)



# Copyright (C) 2014 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero General Public License version 3.0 or later

from __future__ import print_function

import argparse
import glob
import inspect
import os
import sys
from pkg_resources import resource_filename

from .wxf_format import iterate_wxf_tokens
from .compose import compose_svg, cm_to_pixel
from .license import get_license_choices_of_theme, inform_license
from .version import VERSION_STR


_DEFAULT_WIDTH_CM = 7.0

_PIECE_SCALE_MIN = 0.0
_PIECE_SCALE_MAX = 1.2


def _get_self_dir():
    return os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))


def _get_themes_home_dir():
    self_dir = _get_self_dir()
    if os.path.exists(os.path.join(self_dir, '..', '.git')):
        return os.path.join(self_dir, 'themes')
    else:
        return resource_filename('xiangqi_setup', 'themes')


def check(options):
    if _PIECE_SCALE_MIN < options.piece_scale <= _PIECE_SCALE_MAX:
        pass
    else:
        print('ERROR: Piece scale must be larger than %.1f and greater or equal %.1f .' \
                % (_PIECE_SCALE_MIN, _PIECE_SCALE_MAX), file=sys.stderr)
        sys.exit(1)

    if options.width_centimeter is not None:
        options.width_pixel = cm_to_pixel(options.width_centimeter, options.resolution_dpi)
    delattr(options, 'width_centimeter')
    if options.width_pixel is None:
        options.width_pixel = cm_to_pixel(_DEFAULT_WIDTH_CM, options.resolution_dpi)


def run(options):
    pieces_to_put = list(iterate_wxf_tokens(options.input_file))
    compose_svg(pieces_to_put, options)
    inform_license(options.board_theme_dir, options.pieces_theme_dir)


def _theme_name(text):
    if '/' in text:
        raise ValueError('Theme name cannot contain slashes')
    return text

_theme_name.__name__ = 'theme name'  # used by arparse error message


def main():
    themes_home_dir = _get_themes_home_dir()
    board_themes_home_dir = os.path.join(themes_home_dir, 'board')
    pieces_themes_home_dir = os.path.join(themes_home_dir, 'pieces')

    epilog_chunks = []

    # Are we in --help mode (or can we save wasting time collecting all that data)
    if '--help' in sys.argv[1:] or '-h' in sys.argv[1:]:
        board_theme_choices = []
        pieces_theme_choices = []
        for directory, target_list in (
                (board_themes_home_dir, board_theme_choices),
                (pieces_themes_home_dir, pieces_theme_choices),
                ):
            target_list += [os.path.relpath(e, directory) for e in glob.glob(os.path.join(directory, '*', ''))]

        for category, category_home_dir, source_list, blank_line_after in (
                ('available board themes', board_themes_home_dir, board_theme_choices, True),
                ('available pieces themes', pieces_themes_home_dir, pieces_theme_choices, False),
                ):
            epilog_chunks.append('%s (in alphabetic order):' % category)
            for name in sorted(source_list, key=lambda x: x.lower()):
                license_choices = get_license_choices_of_theme(os.path.join(category_home_dir, name))
                epilog_chunks.append('  %-42s (license: %s)' % (name, ' / '.join(license_choices)))
            if blank_line_after:
                epilog_chunks.append('')

    parser = argparse.ArgumentParser(
            epilog='\n'.join(epilog_chunks),
            formatter_class=argparse.RawDescriptionHelpFormatter,
            )

    theme_options = parser.add_argument_group('theme selection')
    theme_options.add_argument('--board', dest='board_theme_dir', metavar='THEME',
            type=_theme_name, default='clean_alpha',
            help='name of board theme to use (default: "%(default)s"; please check the list of available themes below')
    theme_options.add_argument('--pieces', dest='pieces_theme_dir', metavar='THEME',
            type=_theme_name, default='retro_simple',
            help='name of pieces theme to use (default: "%(default)s"; please check the list of available themes below')

    scaling_options = parser.add_argument_group('scaling')
    width_options = scaling_options.add_mutually_exclusive_group()
    width_options.add_argument('--width-px', dest='width_pixel', metavar='PIXEL', type=float,
            help='width of the output in pixels')
    width_options.add_argument('--width-cm', dest='width_centimeter', metavar='CENTIMETER', type=float,
            help='width of the output in centimeters')
    scaling_options.add_argument('--dpi', dest='resolution_dpi', metavar='FLOAT', type=float, default=90.0,
            help='resolution of the output in dots per inch')
    scaling_options.add_argument('--scale-pieces', dest='piece_scale', metavar='FACTOR', type=float, default=0.9,
            help='factor to scale pieces by (%.1f to %.1f, default: %%(default)s)' % (_PIECE_SCALE_MIN, _PIECE_SCALE_MAX))

    parser.add_argument('--debug', action='store_true',
            help='enable debugging (e.g. mark corners of the board)')

    parser.add_argument('input_file', metavar='INPUT_FILE',
            help='location of WXF file to render')
    parser.add_argument('output_file', metavar='OUTPUT_FILE',
            help='location of SVG output file to write')

    parser.add_argument('--version', action='version', version='%(prog)s ' + VERSION_STR)

    options = parser.parse_args()

    # Turn theme names into paths
    options.board_theme_dir = os.path.join(board_themes_home_dir, options.board_theme_dir)
    options.pieces_theme_dir = os.path.join(pieces_themes_home_dir, options.pieces_theme_dir)

    check(options)
    run(options)



# Copyright (C) 2014 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero General Public License version 3.0 or later

from __future__ import print_function

import ConfigParser
import errno
import os

try:
    import svgutils.transform as sg
except ImportError:
    import sys
    print('Please install svg_utils first: https://github.com/btel/svg_utils', file=sys.stderr)
    sys.exit(1)

from .wxf_format import \
        CHARIOT, HORSE, ELEPHANT, ADVISOR, KING, CANNON, PAWN, \
        RED, BLACK 


_BOARD_SVG_BASENAME = 'board.svg'
_BOARD_INI_BASENAME = 'board.ini'
_BOARD_CONFIG_SECTION = 'Board'

_DIAMOND_FILE_NAME = os.path.join('..', 'diamond.svg')

_MAX_X = 8
_MAX_Y = 9


_FILENAME_OF_PARTY_PIECE = {
    RED: {
        CHARIOT: 'red_chariot.svg',
        HORSE: 'red_horse.svg',
        ELEPHANT: 'red_elephant.svg',
        ADVISOR: 'red_advisor.svg',
        KING: 'red_king.svg',
        CANNON: 'red_cannon.svg',
        PAWN: 'red_pawn.svg',
    },
    BLACK: {
        CHARIOT: 'black_chariot.svg',
        HORSE: 'black_horse.svg',
        ELEPHANT: 'black_elephant.svg',
        ADVISOR: 'black_advisor.svg',
        KING: 'black_king.svg',
        CANNON: 'black_cannon.svg',
        PAWN: 'black_pawn.svg',
    }
}


def _length_string_to_pixel(text, resolution_dpi):
    if text.endswith('cm'):
        return cm_to_pixel(float(text[:-2]), resolution_dpi)
    return float(text)

def _cm_to_inch(cm):
    return cm * 0.393700787

def cm_to_pixel(cm, resolution_dpi):
    return _cm_to_inch(cm) * resolution_dpi


def compose_svg(pieces_to_put, options):
    board_svg_filename = os.path.join(options.board_theme_dir, _BOARD_SVG_BASENAME)
    board_ini_filename = os.path.join(options.board_theme_dir, _BOARD_INI_BASENAME)

    # Check for existance ourselves since ConfigParser would throw NoSectionError
    # at us for a missing file.
    if not os.path.exists(board_ini_filename):
        raise IOError(errno.ENOENT, "No such file or directory: '%s'" % board_ini_filename)

    config = ConfigParser.RawConfigParser(defaults={'river': 0.0})
    config.read(board_ini_filename)

    output_board_offset_left_pixel = config.getfloat(_BOARD_CONFIG_SECTION, 'left')
    output_board_offset_top_pixel = config.getfloat(_BOARD_CONFIG_SECTION, 'top')
    output_board_width_pixel = config.getfloat(_BOARD_CONFIG_SECTION, 'width')
    output_board_height_pixel = config.getfloat(_BOARD_CONFIG_SECTION, 'height')
    output_board_river_height_pixel = config.getfloat(_BOARD_CONFIG_SECTION, 'river')

    jobs = []
    for put_piece in pieces_to_put:
        x_rel = float(put_piece.x) / _MAX_X
        y_rel = float(_MAX_Y - put_piece.y) / _MAX_Y
        basename = _FILENAME_OF_PARTY_PIECE[put_piece.party][put_piece.piece]
        filename = os.path.join(options.pieces_theme_dir, basename)
        jobs.append((x_rel, y_rel, filename))

    if options.debug:
        for x_rel in (0.0, 1.0):
            for y_rel in (0.0, 1.0):
                jobs.append((x_rel, y_rel, os.path.join(options.pieces_theme_dir, _DIAMOND_FILE_NAME)))

    # Read board
    board_fig = sg.fromfile(board_svg_filename)
    board_root = board_fig.getroot()

    # Scale board to output
    board_width_pixel, board_height_pixel = [_length_string_to_pixel(e, options.resolution_dpi) for e in board_fig.get_size()]
    height_factor = board_height_pixel / float(board_width_pixel)
    board_scale = options.width_pixel / board_width_pixel
    board_root.moveto(0, 0, scale=board_scale)

    output_board_offset_left_pixel *= board_scale
    output_board_offset_top_pixel *= board_scale
    output_board_width_pixel *= board_scale
    output_board_height_pixel *= board_scale
    output_board_river_height_pixel *= board_scale

    # Initialize output figure
    output_fig = sg.SVGFigure(
            str(options.width_pixel),
            str(options.width_pixel * height_factor))
    output_fig.append([board_root, ])

    for (x_rel, y_rel, filename) in jobs:
        piece_fig = sg.fromfile(filename)
        piece_root = piece_fig.getroot()
        original_piece_width_pixel, original_piece_height_pixel = [
                _length_string_to_pixel(s, options.resolution_dpi)
                for s in piece_fig.get_size()
                ]

        # Scale and put piece onto board
        center_x_pixel = output_board_offset_left_pixel + output_board_width_pixel * x_rel
        center_y_pixel = output_board_offset_top_pixel + (output_board_height_pixel - output_board_river_height_pixel) * y_rel \
                + (output_board_river_height_pixel if (y_rel >= 0.5) else 0.0)

        maximum_future_piece_width_pixel = output_board_width_pixel / _MAX_X * options.piece_scale
        maximum_future_piece_height_pixel = output_board_height_pixel / _MAX_Y * options.piece_scale

        scale = min(maximum_future_piece_width_pixel / original_piece_width_pixel, maximum_future_piece_height_pixel / original_piece_height_pixel)

        future_piece_width_pixel = original_piece_width_pixel * scale
        future_piece_height_pixel = original_piece_height_pixel * scale

        x_pixel = center_x_pixel - future_piece_width_pixel / 2.0
        y_pixel = center_y_pixel - future_piece_height_pixel / 2.0
        piece_root.moveto(x_pixel, y_pixel, scale=scale)
        output_fig.append([piece_root, ])

    output_fig.save(options.output_file)



# Copyright (C) 2015 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero General Public License version 3.0 or later

from __future__ import print_function

import json
import os
from textwrap import dedent


_LICENSE_DETAILS = {
    'CC-BY-4.0': (
        'Creative Commons Attribution 4.0',
        'https://creativecommons.org/licenses/by/4.0/',
        ),
    'CC-BY-SA-4.0': (
        'Creative Commons Attribution-ShareAlike 4.0',
        'https://creativecommons.org/licenses/by-sa/4.0/',
        ),
    'CC0-1.0': (
        'CC0 1.0 Universal (CC0 1.0) Public Domain Dedication',
        'https://creativecommons.org/publicdomain/zero/1.0/',
        ),
    'FDL-1.2+': (
        'GNU Free Documentation License 1.2 or later',
        'https://gnu.org/licenses/fdl.html',
        ),
    'non-commercial': (
        'Non-commercial use only',
        None,
        ),
    'public-domain': (
        'Public domain',
        'https://en.wikipedia.org/wiki/Public_domain',
        )
}


def _get_license_json_path(single_theme_dir):
    return os.path.join(single_theme_dir, 'LICENSE.json')


def _get_license_json(single_theme_dir):
    license_json_path = _get_license_json_path(single_theme_dir)
    f = open(license_json_path, 'r')
    content = f.read()
    try:
        doc = json.loads(content)
    except ValueError as e:
        raise ValueError('%s (file "%s")' % (e, license_json_path))
    f.close()
    return doc['work']


def get_license_choices_of_theme(single_theme_dir):
    top_work = _get_license_json(single_theme_dir)
    try:
        return [top_work['license_id']]
    except KeyError:
        try:
            return top_work['license_ids_any_of']
        except KeyError:
            raise ValueError('Malformed license file "%s"' \
                    % _get_license_json_path(single_theme_dir))


def _describe_license(license_id):
    try:
        long_name, details_url = _LICENSE_DETAILS[license_id]
    except KeyError:
        return license_id

    if details_url is None:
        return long_name
    else:
        return '%s  (%s)' % (long_name, details_url)


def inform_license(board_theme_dir, pieces_theme_dir):
    print('The license of the themes used apply to the generated image.  In detail:')
    print()

    for category, theme_dir in (
            ('Board', board_theme_dir),
            ('Pieces', pieces_theme_dir),
            ):
        top_work = _get_license_json(theme_dir)


        author_chunks = []
        for author_dict in top_work['authors']:
            for author_name, details_dict in author_dict.items():
                contact_infos = []

                if 'website' in details_dict:
                    contact_infos.append(details_dict['website'])

                if 'email' in details_dict:
                    contact_infos.append('%s@%s' % tuple(details_dict['email']))

                if contact_infos:
                    author_display = '%s (%s)' % (author_name, ', '.join(contact_infos))
                else:
                    author_display = author_name

                author_chunks.append(author_display)
        authors = '\n'.join(('    ' + e) for e in author_chunks)


        try:
            license_ids_any_of = [top_work['license_id']]
        except KeyError:
            try:
                license_ids_any_of = top_work['license_ids_any_of']
            except KeyError:
                raise ValueError('Malformed license file "%s"' % _get_license_json_path(theme_dir))

        if len(license_ids_any_of) == 1:
            license_options = '    %s' % _describe_license(license_ids_any_of[0])
        else:
            license_options = '\n'.join(
                    ('    %s) %s' % (chr(ord('a') + i), _describe_license(e))) \
                    for i, e \
                    in enumerate(license_ids_any_of))


        print(dedent("""\
            %s theme:
              Author(s):
            %s
              License options:
            %s
            """) % (category, authors, license_options))

    print('If this license does not work for you, please pick a different board theme and/or pieces theme.'
        '  '
        'Thank you!')



# Copyright (C) 2014 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero General Public License version 3.0 or later

from __future__ import print_function

import argparse

from .painter import BoardPainter


def run(options):
    board = BoardPainter(
            options.line_thickness_px,
            options.field_width_px,
            options.field_height_px,
            options.border_thickness_px,
            options.border_gap_width_px,
            options.border_gap_height_px,
            options.cross_width_px,
            options.cross_thickness_px,
            options.cross_gap_px,
            )
    board.draw()
    board.write_svg(options.svg_output_file)
    board.write_board_ini(options.ini_output_file)


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--line-thickness-px', metavar='FLOAT', default=1.0, type=float,
            help='Line thickness of square fields in pixel (default: %(default)d)')
    parser.add_argument('--field-width-px', metavar='FLOAT', default=53.0, type=float,
            help='Width of fields in pixel (default: %(default)d)')
    parser.add_argument('--field-height-px', metavar='FLOAT', default=53.0, type=float,
            help='Height of fields in pixel (default: %(default)d)')

    parser.add_argument('--border-thickness-px', metavar='FLOAT', default=2.0, type=float,
            help='Line thickness of border in pixel (default: %(default)d)')
    parser.add_argument('--border-gap-width-px', metavar='FLOAT', default=40.0, type=float,
            help='Widtn of gap to border in pixel (default: %(default)d)')
    parser.add_argument('--border-gap-height-px', metavar='FLOAT', default=40.0, type=float,
            help='Height of gap to border in pixel (default: %(default)d)')

    parser.add_argument('--cross-width-px', metavar='FLOAT', default=10.0, type=float,
            help='Width of starting position cross segments in pixel (default: %(default)d)')
    parser.add_argument('--cross-thickness-px', metavar='FLOAT', default=1.0, type=float,
            help='Line thickness of starting position cross in pixel (default: %(default)d)')
    parser.add_argument('--cross-gap-px', metavar='FLOAT', default=4.0, type=float,
            help='Gap to starting position cross in pixel (default: %(default)d)')

    parser.add_argument('svg_output_file', metavar='SVG_FILE')
    parser.add_argument('ini_output_file', metavar='INI_FILE')
    options = parser.parse_args()
    run(options)



#! /usr/bin/env python2
# Copyright (C) 2015 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero General Public License version 3.0 or later

import os
from setuptools import find_packages, setup

from xiangqi_setup.version import VERSION_STR


def _generate_package_data():
    return {
        root.replace(os.sep, '.'): files
        for root, dirs, files
        in os.walk('xiangqi_setup/themes')
    }


if __name__ == '__main__':
    setup(
            name='xiangqi-setup',
            description='Command line tool to generate razor-sharp Xiangqi (Chinese chess) setup graphics',
            license='GNU Affero General Public License version 3.0 or later',
            version=VERSION_STR,
            author='Sebastian Pipping',
            author_email='sebastian@pipping.org',
            url='https://github.com/hartwork/xiangqi-setup',
            download_url='https://github.com/hartwork/xiangqi-setup/archive/%s.tar.gz' % VERSION_STR,
            packages=find_packages(),
            package_data=_generate_package_data(),
            scripts=[
                'xiangqi-board',
                'xiangqi-setup',
            ],
            install_requires=[
                'svgutils>=0.3.1',
            ],
            classifiers=[
                'Development Status :: 4 - Beta',
                'Environment :: Console',
                'Intended Audience :: Education',
                'Intended Audience :: Other Audience',
                'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
                'Natural Language :: English',
                'Programming Language :: Python',
                'Programming Language :: Python :: 2',
                'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 2 :: Only',
                'Topic :: Artistic Software',
                'Topic :: Games/Entertainment :: Board Games',
                'Topic :: Multimedia :: Graphics',
                'Topic :: Printing',
                'Topic :: Utilities',
            ],
    )



# Copyright (C) 2014 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero General Public License version 3.0 or later

from __future__ import print_function

import ConfigParser

import svgutils.transform as sg


class BoardPainter(object):
    def __init__(self,
            line_thickness_pixel,
            field_width_px,
            field_height_px,
            border_thickness_pixel,
            border_gap_width_pixel,
            border_gap_height_pixel,
            cross_length_pixel,
            cross_thickness_pixel,
            cross_gap_pixel,
            ):
        self._lines = []

        self._line_thickness_pixel = line_thickness_pixel
        self._field_width_px = field_width_px
        self._field_height_px = field_height_px
        self._border_thickness_pixel = border_thickness_pixel
        self._border_gap_width_pixel = border_gap_width_pixel
        self._border_gap_height_pixel = border_gap_height_pixel
        self._cross_width_pixel = cross_length_pixel
        self._cross_thickness_pixel = cross_thickness_pixel
        self._cross_gap_pixel = cross_gap_pixel

    def _playing_field_offset_left(self):
        return self._border_thickness_pixel + self._border_gap_width_pixel + self._line_thickness_pixel / 2.0

    def _playing_field_offset_top(self):
        return self._border_thickness_pixel + self._border_gap_height_pixel + self._line_thickness_pixel / 2.0

    def _cross(self, (column, row)):
        x = self._playing_field_offset_left() + column * self._field_width_px
        y = self._playing_field_offset_top() + row * self._field_height_px
        return (x, y)

    def _raw_line(self, start, end, width):
        line = sg.LineElement([start, end], width=width)
        self._lines.append(line)

    def _grid_line(self, start, end):
        self._raw_line(self._cross(start), self._cross(end), self._line_thickness_pixel)

    def _outer_board_width_pixel(self):
        return 2 * self._border_thickness_pixel + 2 * self._border_gap_width_pixel + self._line_thickness_pixel \
                + 8 * self._field_width_px

    def _outer_board_height_pixel(self):
        return 2 * self._border_thickness_pixel + 2 * self._border_gap_height_pixel + self._line_thickness_pixel \
                + 9 * self._field_height_px

    def _draw_border(self):
        # Vertical lines
        WIDTH = self._outer_board_width_pixel()
        HEIGHT = self._outer_board_height_pixel()
        X_LEFT_CENTER = self._border_thickness_pixel / 2.0
        X_RIGHT_CENTER = WIDTH - self._border_thickness_pixel / 2.0
        Y_TOP_CENTER = self._border_thickness_pixel / 2.0
        Y_BOTTOM_CENTER = HEIGHT - self._border_thickness_pixel / 2.0
        for start, end in (
                # Vertical lines
                ((X_LEFT_CENTER, 0), (X_LEFT_CENTER, HEIGHT)),
                ((X_RIGHT_CENTER, 0), (X_RIGHT_CENTER, HEIGHT)),

                # Horizontal lines
                ((0, Y_TOP_CENTER), (WIDTH, Y_TOP_CENTER)),
                ((0, Y_BOTTOM_CENTER), (WIDTH, Y_BOTTOM_CENTER)),
                ):
            line = sg.LineElement([start, end], width=self._border_thickness_pixel)
            self._lines.append(line)

    def _draw_playing_field(self):
        # Vertical lines, river gap
        for column in range(0, 9):
            if 0 < column < 8:
                self._grid_line((column, 0), (column, 4))
                self._grid_line((column, 5), (column, 9))
            else:
                self._grid_line((column, 0), (column, 9))

        # Horizontal lines
        for row in range(0, 10):
            self._grid_line((0, row), (8, row))

        # Palace crosses
        self._grid_line((3, 0), (5, 2))
        self._grid_line((3, 2), (5, 0))
        self._grid_line((3, 7), (5, 9))
        self._grid_line((3, 9), (5, 7))

    def _add_setup_helper(self, column, row, left, right):
        assert left or right
        center = self._cross((column, row))
        for x_factor, y_factor, wanted in (
                (1, -1, right),  # top right
                (1, 1, right),  # bottom right
                (-1, 1, left),  # bottom left
                (-1, -1, left),  # top left
                ):
            if not wanted:
                continue

            GAP = self._line_thickness_pixel / 2.0 + self._cross_gap_pixel + self._cross_thickness_pixel / 2.0
            start_x = center[0] + x_factor * GAP
            start_y = center[1] + y_factor * GAP

            # Vertical part
            end_x = start_x
            end_y = start_y + y_factor * self._cross_width_pixel
            SHIFT_Y = y_factor * self._cross_thickness_pixel / 2.0
            self._raw_line((start_x, start_y - SHIFT_Y), (end_x, end_y - SHIFT_Y), self._cross_thickness_pixel)

            # Horizontal part
            end_x = start_x + x_factor * self._cross_width_pixel
            end_y = start_y
            SHIFT_X = x_factor * self._cross_thickness_pixel / 2.0
            self._raw_line((start_x - SHIFT_X, start_y), (end_x - SHIFT_X, end_y), self._cross_thickness_pixel)

    def _draw_setup_helpers(self):
        for column, row in (
                # Black cannons
                (1, 2), (7, 2),
                # Black pawns
                (0, 3),  (2, 3), (4, 3), (6, 3), (8, 3),
                # Red pawns
                (0, 6),  (2, 6), (4, 6), (6, 6), (8, 6),
                # Red cannons
                (1, 7), (7, 7),
                ):
            left = (column > 0)
            right = (column < 8)
            self._add_setup_helper(column, row, left, right)

    def draw(self):
        self._draw_playing_field()
        if self._cross_thickness_pixel != 0:
            self._draw_setup_helpers()
        if self._border_thickness_pixel != 0:
            self._draw_border()

    def write_svg(self, filename):
        WIDTH = self._outer_board_width_pixel()
        HEIGHT = self._outer_board_height_pixel()
        output_fig = sg.SVGFigure(str(WIDTH), str(HEIGHT))
        output_fig.append(self._lines)
        output_fig.save(filename)

    def write_board_ini(self, filename):
        f = open(filename, 'w')
        config = ConfigParser.RawConfigParser()
        config.add_section('Board')
        config.set('Board', 'left', str(self._playing_field_offset_left()))
        config.set('Board', 'top', str(self._playing_field_offset_top()))
        config.set('Board', 'width', str(self._field_width_px * 8))
        config.set('Board', 'height', str(self._field_height_px * 9))
        config.write(f)
        f.close()
